#' batea_isor_3p

#' batea_isor_3p
#' @param dataset list or tibble
#' @param m_tilde name of the column representative of the prediction
#' @param x_tilde name of the column representative of the input
#' @param y_tilde name of the column representative of the output
#' @param phi_f1  hyper-parameters of phi_f1
#' @param phi_f2  hyper-parameters of phi_f2
#' @param phi_f3  hyper-parameters of phi_f3
#' @param phi_g1  hyper-parameters of phi_g1
#' @param phi_g2  hyper-parameters of phi_g2
#' @param phi_g3  hyper-parameters of phi_g3
#' @param phi_h1  hyper-parameters of phi_h1
#' @param beta_r  hyper-parameters of beta_r
#' @param sig_r  hyper-parameters of sig_r
#' @param FUN FUN function for the prediction
#' @param alg chosen algorithm for mcmc
#' @param niter number of iteration
#' @keywords batqa
#' @export
#' @details Compute the batea assuming only output are non-true
#' @examples
#' \dontrun{
#' batea_isor_3p()
#' }
batea_isor_3p <- function(dataset,m_tilde,x_tilde,y_tilde,phi_f1,phi_f2,phi_f3,phi_g1,phi_g2,phi_g3,phi_h1,beta_r,sig_r,FUN,alg="mh_pw",niter=1000){

  inc <- 0
  # prior
  prior_ <- bato::prior(

    name = c("phi_f1","phi_f2","phi_f3","phi_g1","phi_g2","phi_g3","phi_h1","beta_r","sig_r"),

    hyperpar = function(){
      res <- list(
          phi_f1 = phi_f1,
          phi_f2 = phi_f2,
          phi_f3 = phi_f3,
          phi_g1 = phi_g1,
          phi_g2 = phi_g2,
          phi_g3 = phi_g3,
          phi_h1 = phi_h1,
          beta_r = beta_r,
          sig_r  = sig_r)

       return(res)
     },

     init = function(hyperpar){
       res <- list(
         phi_f1 = hyperpar[["phi_f1"]]$mean,
         phi_f2 = hyperpar[["phi_f2"]]$mean,
         phi_f3 = hyperpar[["phi_f3"]]$mean,
         phi_g1 = hyperpar[["phi_g1"]]$mean,
         phi_g2 = hyperpar[["phi_g2"]]$mean,
         phi_g3 = hyperpar[["phi_g3"]]$mean,
         phi_h1 = hyperpar[["phi_h1"]]$mean,
         beta_r = hyperpar[["beta_r"]]$mean,
         sig_r  = hyperpar[["sig_r"]]$df)
       return(res)
     },

    density = function(x, v, hyperpar){
      d_phi_f1 = dnorm(x[["phi_f1"]], mean= v[["phi_f1"]], sd = hyperpar[["phi_f1"]]$sd, log =TRUE)
      d_phi_f2 = dnorm(x[["phi_f2"]], mean= v[["phi_f2"]], sd = hyperpar[["phi_f2"]]$sd, log =TRUE)
      d_phi_f3 = dnorm(x[["phi_f3"]], mean= v[["phi_f3"]], sd = hyperpar[["phi_f3"]]$sd, log =TRUE)
      d_phi_g1 = dnorm(x[["phi_g1"]], mean= v[["phi_g1"]], sd = hyperpar[["phi_g1"]]$sd, log =TRUE)
      d_phi_g2 = dnorm(x[["phi_g2"]], mean= v[["phi_g2"]], sd = hyperpar[["phi_g2"]]$sd, log =TRUE)
      d_phi_g3 = dnorm(x[["phi_g3"]], mean= v[["phi_g3"]], sd = hyperpar[["phi_g3"]]$sd, log =TRUE)
      d_phi_h1 = dnorm(x[["phi_h1"]], mean= v[["phi_h1"]], sd = hyperpar[["phi_h1"]]$sd, log =TRUE)
      d_beta_r = dnorm(x[["beta_r"]], mean= v[["beta_r"]], sd = hyperpar[["beta_r"]]$sd, log =TRUE)
      d_sig_r = dinvchisq(x[["sig_r"]], df=hyperpar[["sig_r"]]$df, scale=1/(v[["sig_r"]]^2), LOG = TRUE)
      return(d_phi_f1 + d_phi_f2 + d_phi_f3 + d_phi_g1 + d_phi_g2 + d_phi_g3 + d_phi_h1 + d_beta_r + d_sig_r)
    },

    sampler = function(n, v, hyperpar){
      r_phi_f1 <- rnorm(n, mean= v[["phi_f1"]], sd = hyperpar[["phi_f1"]]$sd)
      r_phi_f2 <- rnorm(n, mean= v[["phi_f2"]], sd = hyperpar[["phi_f2"]]$sd)
      r_phi_f3 <- rnorm(n, mean= v[["phi_f3"]], sd = hyperpar[["phi_f3"]]$sd)
      r_phi_g1 <- rnorm(n, mean= v[["phi_g1"]], sd = hyperpar[["phi_g1"]]$sd)
      r_phi_g2 <- rnorm(n, mean= v[["phi_g2"]], sd = hyperpar[["phi_g2"]]$sd)
      r_phi_g3 <- rnorm(n, mean= v[["phi_g3"]], sd = hyperpar[["phi_g3"]]$sd)
      r_phi_h1 <- rnorm(n, mean= v[["phi_h1"]], sd = hyperpar[["phi_h1"]]$sd)
      r_beta_r <- rnorm(n, mean= v[["beta_r"]], sd = hyperpar[["beta_r"]]$sd)
      r_sig_r <- rinvchisq(n=n,df=hyperpar[["sig_r"]]$df,scale=1/(v[["sig_r"]]^2))
      res <- list()
      res[["phi_f1"]] <- abs(r_phi_f1)
      res[["phi_f2"]] <- abs(r_phi_f2)
      res[["phi_f3"]] <- abs(r_phi_f3)
      res[["phi_g1"]] <- abs(r_phi_g1)
      res[["phi_g2"]] <- abs(r_phi_g2)
      res[["phi_g3"]] <- abs(r_phi_g3)
      res[["phi_h1"]] <- abs(r_phi_h1)
      res[["beta_r"]] <- r_beta_r
      res[["sig_r"]] <- r_sig_r
      return(res)
    }
  )

  # bato object
  # remark: any kind of loop (if,for,...) must be avoided
  hello <- bato::bato(

      mcmc_par <- list(alg=alg,niter=niter),

      prior = prior_,

      parameters = function(dataset,m_tilde,x_tilde,y_tilde,v,n=1,prior,vec=NULL) {

        prior$sampler(n=n,v=v,hyperpar=prior$hyperpar())
        prior$piece(vec=vec)

        sig_f <- lapply(x_tilde,function(x) expr_sig_f_ts_3par(x_tilde_=dataset[[x]],phi_f1=prior$get_gen()[["phi_f1"]],phi_f2=prior$get_gen()[["phi_f2"]],phi_f3=prior$get_gen()[["phi_f3"]]))
        names(sig_f) <- x_tilde
        sig_g <- expr_sig_g_ts_3par(y_tilde_=dataset[[y_tilde]],phi_g1=prior$get_gen()[["phi_g1"]],phi_g2=prior$get_gen()[["phi_g2"]],phi_g3=prior$get_gen()[["phi_g3"]])
        sig_h <- expr_sig_h_ts_1par(m_tilde_=dataset[[m_tilde]],phi_h1=prior$get_gen()[["phi_h1"]])

        res <- list()
        res[["sig_f"]] <- sig_f
        res[["sig_g"]] <- sig_g
        res[["sig_h"]] <- sig_h
        res[["sig_r"]] <- prior$get_gen()[["sig_r"]]
        res[["beta_r"]] <- prior$get_gen()[["beta_r"]]

        # new x _tilde
        n <- length(res[["sig_f"]])
        x_new <- lapply(x_tilde,function(x) {
          res <- rnorm(n=n,mean=dataset[[x]],sd=res[["sig_f"]][[x]])
          return(res)
        })
        names(x_new) <- x_tilde
        res[["x_new"]] <- x_new
        ## new m _tilde
        res[["m_tilde_new"]] <- FUN(x_new)*res[["beta_r"]]

        return(res)
      },

      likelihoods = function(dataset,m_tilde,x_tilde,y_tilde,v,param,prior,vec=NULL){

        par <- param(dataset=dataset,m_tilde=m_tilde,x_tilde=x_tilde,y_tilde=y_tilde,v=v,prior=prior,vec=vec)

        # likelihoods
        ll1 <- dnorm(x=dataset[[y_tilde]], mean=par[["m_tilde_new"]],sd=sqrt(par[["sig_g"]]^2+par[["sig_h"]]^2+par[["sig_r"]]^2),log = TRUE)

        ll2 <- lapply(x_tilde,function(x) {
          res <- dnorm(x=par[["x_new"]][[x]],mean=dataset[[x]],sd=par[["sig_f"]][[x]],log=TRUE)
          return(res)
        })

        res <- sum(ll1,unlist(ll2),na.rm=TRUE)

        return(res)
      }

  )

  # process
  hello$mcmc(dataset=dataset,m_tilde=m_tilde,x_tilde=x_tilde,y_tilde=y_tilde)

  # results
  res <- list(
    type = "batea_isor",
    time_lag = hello$get_time_lag(),
    name = c("phi_f1","phi_f2","phi_f3","phi_g1","phi_g2","phi_g3","phi_h1","beta_r","sig_r"),
    distrib = c("norm","norm","norm","norm","norm","norm","norm","norm","invchisq"),
    hyperpar = list(phi_f1=phi_f1,phi_f2=phi_f2,phi_f3=phi_f3,phi_g1=phi_g1,phi_g2=phi_g2,phi_g3=phi_g3,phi_h1=phi_h1,beta_r=beta_r,sig_r=sig_r),
    phi = hello$get_prior_gen(),
    al = hello$get_alpha(),
    mcmc_par = list(alg=alg,niter=niter)
    # m_tilde_verbose = m_tilde_verbose
  )

  class(res) <- c("batea",class(res))

  return(res)




}
