
#' batea_or_3p

#' batea_or_3p
#' @param dataset list or tibble
#' @param m_tilde name of the column representative of the prediction
#' @param y_tilde name of the column representative of the output
#' @param phi_g1  hyper-parameters of phi_g1
#' @param phi_g2  hyper-parameters of phi_g2
#' @param phi_g3  hyper-parameters of phi_g3
#' @param beta_r  hyper-parameters of beta_r
#' @param sig_r  hyper-parameters of sig_r
#' @param alg chosen algorithm for mcmc
#' @param niter number of iteration
#' @keywords batqa
#' @export
#' @details Compute the batea assuming only output are non-true
#' @examples
#' \dontrun{
#' batea_or()
#' }
batea_or_3p <- function(dataset,m_tilde,y_tilde,phi_g1,phi_g2,phi_g3,beta_r,sig_r,alg="mh_pw",niter=1000){

  # prior
  prior_ <- bato::prior(

    name = c("phi_g1","phi_g2","phi_g3","beta_r","sig_r"),

    hyperpar = function(){
      res <- list(
          phi_g1 = phi_g1,
          phi_g2 = phi_g2,
          phi_g3 = phi_g3,
          beta_r = beta_r,
          sig_r  = sig_r)

       return(res)
     },

     init = function(hyperpar){
       res <- list(
         phi_g1 = hyperpar[["phi_g1"]]$mean,
         phi_g2 = hyperpar[["phi_g2"]]$mean,
         phi_g3 = hyperpar[["phi_g3"]]$mean,
         beta_r = hyperpar[["beta_r"]]$mean,
         sig_r  = hyperpar[["sig_r"]]$df
       )
       return(res)
     },

    density = function(x,v, hyperpar){
       d_phi_g1 = dnorm(x[["phi_g1"]], mean= v[["phi_g1"]], sd = hyperpar[["phi_g1"]]$sd, log =TRUE)
       d_phi_g2 = dnorm(x[["phi_g2"]], mean= v[["phi_g2"]], sd = hyperpar[["phi_g2"]]$sd, log =TRUE)
       d_phi_g3 = dnorm(x[["phi_g2"]], mean= v[["phi_g3"]], sd = hyperpar[["phi_g3"]]$sd, log =TRUE)
       d_beta_r = dnorm(x[["beta_r"]], mean= v[["beta_r"]], sd = hyperpar[["beta_r"]]$sd, log =TRUE)
       d_sig_r = dinvchisq(x[["sig_r"]], df=hyperpar[["sig_r"]]$df, scale=1/(v[["sig_r"]]^2), LOG = TRUE)
       return(d_phi_g1 + d_phi_g2 + d_phi_g3 + d_beta_r + d_sig_r)
    },

    sampler = function(n,v,hyperpar){
      r_phi_g1 <- rnorm(n, mean= v[["phi_g1"]], sd = hyperpar[["phi_g1"]]$sd)
      r_phi_g2 <- rnorm(n, mean= v[["phi_g2"]], sd = hyperpar[["phi_g2"]]$sd)
      r_phi_g3 <- rnorm(n, mean= v[["phi_g3"]], sd = hyperpar[["phi_g3"]]$sd)
      r_beta_r <- rnorm(n, mean= v[["beta_r"]], sd = hyperpar[["beta_r"]]$sd)
      r_sig_r <- rinvchisq(n=n,df=hyperpar[["sig_r"]]$df,scale=1/(v[["sig_r"]]^2))
      res <- list()
      res[["phi_g1"]] <- abs(r_phi_g1)
      res[["phi_g2"]] <- abs(r_phi_g2)
      res[["phi_g3"]] <- abs(r_phi_g3)
      res[["beta_r"]] <- r_beta_r
      res[["sig_r"]] <- r_sig_r
      return(res)
    }
  )

  # bato object
  # remark: any kind of loop (if,for,...) must be avoided
  hello <- bato::bato(

      mcmc_par <- list(alg=alg,niter=niter),

      prior = prior_,

      parameters = function(dataset,y_tilde,m_tilde,v,n=1,prior,vec=NULL) {
        res <- list()
        prior$sampler(n=n,v=v,hyperpar=prior$hyperpar())
        prior$piece(vec=vec)
        sig_g <- expr_sig_g_ts_3par(y_tilde_=dataset[[y_tilde]],phi_g1=prior$get_gen()[["phi_g1"]],phi_g2=prior$get_gen()[["phi_g2"]],phi_g3=prior$get_gen()[["phi_g3"]])
        res[["sig_g"]] <- sig_g
        res[["sig_r"]] <- prior$get_gen()[["sig_r"]]
        res[["beta_r"]] <- prior$get_gen()[["beta_r"]]
        ## new m _tilde
        res[["m_tilde_new"]] <- dataset[[m_tilde]]*res[["beta_r"]]


        return(res)
      },

      likelihoods = function(dataset,m_tilde,y_tilde,v,param,prior,vec=NULL){
        par <- param(dataset=dataset,y_tilde=y_tilde,m_tilde=m_tilde,v=v,prior=prior,vec=vec)

        # likelihoods
        ll1 <- dnorm(x=dataset[[y_tilde]], mean=par[["m_tilde_new"]],sd=sqrt(par[["sig_g"]]^2+par[["sig_r"]]^2),log = TRUE)

        res <- sum(ll1,na.rm=TRUE)

        return(res)
      }

  )

  # process
  hello$mcmc(dataset=dataset,m_tilde=m_tilde,y_tilde=y_tilde)

  # results
  res <- list(
    time_lag = hello$get_time_lag(),
    type = "batea_or",
    name = c("phi_g1","phi_g2","phi_g3","beta_r","sig_r"),
    distrib = c("norm","norm","norm","norm","invchisq"),
    hyperpar = list(phi_g1=phi_g1,phi_g2=phi_g2,phi_g3=phi_g3,beta_r=beta_r,sig_r=sig_r),
    phi = hello$get_prior_gen(),
    al = hello$get_alpha(),
    mcmc_par = list(alg=alg,niter=niter)
  )

  class(res) <- c("batea",class(res))

  return(res)


}
