#' batea_r

#' batea_r
#' @param dataset list or tibble
#' @param m_tilde name of the column representative of the prediction
#' @param y_tilde name of the column representative of the output
#' @param beta_r  hyper-parameters of beta_r
#' @param sig_r  hyper-parameters of sig_r
#' @param alg chosen algorithm for mcmc
#' @param niter number of iteration
#' @keywords batqa
#' @export
#' @details Compute the batea assuming only output are non-true
#' @examples
#' \dontrun{
#' batea_r()
#' }
batea_r <- function(dataset,m_tilde,y_tilde,beta_r,sig_r,alg="mh_pw",niter=1000){

  # prior
  prior_ <- bato::prior(

    name = c("beta_r","sig_r"),

    hyperpar = function(){
      res <- list(
          beta_r = beta_r,
          sig_r = sig_r)

       return(res)
     },

     init = function(hyperpar){
       res <- list(
         beta_r = hyperpar[["beta_r"]]$mean,
         sig_r = hyperpar[["sig_r"]]$df
       )
       return(res)
     },

    density = function(x,v, hyperpar){
       d_beta_r = dnorm(x[["beta_r"]], mean= v[["beta_r"]], sd = hyperpar[["beta_r"]]$sd, log =TRUE)
       d_sig_r = dinvchisq(x[["sig_r"]], df=hyperpar[["sig_r"]]$df, scale=1/(v[["sig_r"]]^2), LOG = TRUE)
       return(d_beta_r + d_sig_r)
    },

    sampler = function(n,v,hyperpar){
      r_beta_r <- rnorm(n, mean= v[["beta_r"]], sd = hyperpar[["beta_r"]]$sd)
      r_sig_r <- rinvchisq(n=n,df=hyperpar[["sig_r"]]$df,scale=1/(v[["sig_r"]]^2))
      res <- list()
      res[["beta_r"]] <- r_beta_r
      res[["sig_r"]] <- r_sig_r
      return(res)
    }
  )

  # bato object
  # remark: any kind of loop (if,for,...) must be avoided
  hello <- bato::bato(

      mcmc_par <- list(alg=alg,niter=niter),

      prior = prior_,

      parameters = function(dataset,m_tilde,v,n=1,prior,vec=NULL) {
        res <- list()
        prior$sampler(n=n,v=v,hyperpar=prior$hyperpar())
        prior$piece(vec=vec)
        res[["sig_r"]] <- prior$get_gen()[["sig_r"]]
        res[["beta_r"]] <- prior$get_gen()[["beta_r"]]
        ## new m _tilde
        res[["m_tilde_new"]] <- dataset[[m_tilde]]*res[["beta_r"]]
        return(res)
      },

      likelihoods = function(dataset,m_tilde,y_tilde,v,param,prior,vec=NULL){
        par <- param(dataset=dataset,m_tilde=m_tilde,v=v,prior=prior,vec=vec)

        # likelihoods
        ll1 <- dnorm(x=dataset[[y_tilde]], mean=par[["m_tilde_new"]],sd=par[["sig_r"]],log = TRUE)
        res <- sum(ll1,na.rm=TRUE)
        return(res)
      }

  )

  # process
  hello$mcmc(dataset=dataset,m_tilde=m_tilde,y_tilde=y_tilde)

  # results
  res <- list(
    type = "batea_r",
    time_lag = hello$get_time_lag(),
    name = c("beta_r","sig_r"),
    distrib = c("norm","invchisq"),
    hyperpar = list(beta_r=beta_r,sig_r=sig_r),
    phi = hello$get_prior_gen(),
    al = hello$get_alpha(),
    mcmc_par = list(alg=alg,niter=niter)
  )

  class(res) <- c("batea",class(res))

  return(res)


}
