#' fit_batea

#' fit_batea
#' @param train train dataset list or tibble
#' @param test test dataset list or tibble
#' @param target column name of the target
#' @param predictors column names of the predictors
#' @param FUN_fit function for the fitting
#' @param FUN_predict function for the prediciton
#' @param batea_type chosen BATEA type 'r','or','sor','isor'
#' @param alg algorithm name for the BATEA optimisation
#' @param niter number of iteration for the optimisation
#' @param hyperpar hyper-parameters of the a prior information about input, output, structure, and remnant errors
#' @param burnin burnin percentage by default 30
#' @param BURNIN boolean by default TRUE
#' @keywords batqa
#' @export
#' @examples
#' \dontrun{
#' fit_batea()
#' }
#
fit_batea <- function(train,test,target,predictors,FUN_fit,FUN_predict,
                                                batea_type=c("r","or","sor","isor"),
                                                alg="mh_pw",niter=50000,
                                                hyperpar=list(
                                                      phi_f1 = list(mean=4.35/100/1.96,sd=3/100),
                                                      phi_g1 = list(mean=4.35/100/1.96,sd=3/100),
                                                      phi_h1 = list(mean=0.1,sd=0.1),
                                                      beta_r  = list(mean=1,sd=2),
                                                      sig_r  = list(df=1,scale=0.2)),
                                                burnin=30,
                                                BURNIN=TRUE){

  ############################
  # fit, predict dataset     #
  ############################
  tmp_model <- FUN_fit(dataset=train,target=target,predictors=predictors)
  tmp_pred <- FUN_predict(object=tmp_model,dataset=test)
  dataset <- dplyr::bind_cols(test,prediction=tmp_pred)

  ################
  # OPTIM BATEA  #
  ################

  res_batea <- switch(batea_type,

  # with remnant error
  "r" =  batqa::batea_r(dataset=dataset,m_tilde="prediction",y_tilde=target,beta_r = hyperpar[["beta_r"]],sig_r = hyperpar[["sig_r"]], alg=alg,niter=niter),


  # with output and remnant error
  "or" =  batqa::batea_or_1p(dataset=dataset,m_tilde="prediction",y_tilde=target,
                            phi_g1 = hyperpar[["phi_g1"]],
                            beta_r = hyperpar[["beta_r"]],
                            sig_r = hyperpar[["sig_r"]],
                            alg=alg,niter=niter),

  # with structure, output and remnant error
  "sor" = batqa::batea_sor_1p(dataset=dataset,m_tilde="prediction",y_tilde=target,
                            phi_g1 = hyperpar[["phi_g1"]],
                            phi_h1 = hyperpar[["phi_h1"]],
                            beta_r = hyperpar[["beta_r"]],
                            sig_r = hyperpar[["sig_r"]],
                            alg=alg,niter=niter),

  # with input, structure, output and remnant error
  "isor" = batqa::batea_isor_1p(dataset=dataset,m_tilde="prediction",x_tilde=predictors,y_tilde=target,
                            phi_f1 = hyperpar[["phi_f1"]],
                            phi_g1 = hyperpar[["phi_g1"]],
                            phi_h1 = hyperpar[["phi_h1"]],
                            beta_r = hyperpar[["beta_r"]],
                            sig_r = hyperpar[["sig_r"]],
                            FUN = function(x){
                              res <- FUN_predict(object=tmp_model,x)
                              return(res)
                            },
                            alg=alg,niter=niter),

  stop("'batea_type' not recognized. Method be chosen between 'r', 'or', 'sor', 'isor'."))

  #########################
  # POSTERIOR ESTIMATION  #
  #########################

#!!!!!
#  xposterior <- batqa::get_posterior(res_batea,BURNIN=FALSE)
#
#  for (i in 1:length(res_batea$name)){
#    print(paste0("\n", res_batea$name[i], " min:", min(xposterior[[res_batea$name[i]]],na.rm=TRUE),", max:", max(xposterior[[res_batea$name[i]]],na.rm=TRUE)))
#  }
#  rm(xposterior)
#!!!!!

  post_est <- tryCatch({
    res <- list(values = estimate_posterior(res_batea,burnin=burnin,BURNIN=BURNIN),error=NULL)
    res
  }, error = function(err) {
    res <- list(values=NULL,error=as.character(err))
    res
  })

  res <- list(core = tmp_model,
              batea = res_batea,
              post_est = post_est,
              x_tilde = predictors,
              y_tilde = target,
              m_tilde = "prediction")

  class(res) <- c("model_xBATEA",class(res))
  return(res)

}
